'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');
var _utils = require('@utils');

/**
 * Enhanced Tokyo Night Day theme color definitions
 * ---------------------------------------------
 * Colors organized by function with visual color blocks
 */
// Base colors
const base00 = '#e1e2e7'; // Background
const base01 = '#3760bf'; // Primary foreground
const base02 = '#99a7df'; // Selection background
const base03 = '#848cb5'; // Comments, invisibles
const base04 = '#8c91a8'; // Dark foreground (status)
const base05 = '#3760bf'; // Default foreground
const base06 = '#e9e9ec'; // Light background
const base07 = '#d5d6db'; // Light background (gutter)
// Accent colors
const base_red = '#f52a65'; // Errors, invalid
const base_orange = '#b15c00'; // Numbers, constants
const base_yellow = '#8c6c3e'; // Classes, attributes
const base_green = '#587539'; // Strings, success
const base_cyan = '#007197'; // Functions, keywords
const base_blue = '#2e7de9'; // Variables, parameters
const base_purple = '#7847bd'; // Operators, tags
const base_magenta = '#9854f1'; // Special characters
// UI specific colors
const invalid = base_red;
const darkBackground = base07;
const highlightBackground = '#5F5FAF5A'; // Line highlight with improved transparency
const background = base00;
const tooltipBackground = base06;
const selection = '#99a7df40'; // Selection background with transparency
const selectionMatch = '#99a7df60'; // Selection match with transparency
const cursor = base01; // Cursor color
const activeBracketBg = '#0e639c20'; // Active bracket background with transparency
const activeBracketBorder = base_cyan; // Active bracket border
const diagnosticWarning = base_orange; // Warning color
const linkColor = base_cyan; // Link color
const visitedLinkColor = base_purple; // Visited link color
/**
 * Enhanced editor theme styles for Tokyo Night Day
 */
const tokyoNightDayTheme = view.EditorView.theme({
    // Base editor styles
    '&': {
        color: base05,
        backgroundColor: background,
        fontSize: _utils.generalContent.fontSize,
        fontFamily: _utils.generalContent.fontFamily,
    },
    // Content and cursor
    '.cm-content': {
        caretColor: cursor,
        lineHeight: _utils.generalContent.lineHeight,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
        borderLeftWidth: _utils.generalCursor.borderLeftWidth,
    },
    '.cm-fat-cursor': {
        backgroundColor: `${cursor}99`,
        color: background,
    },
    // Selection
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    // Make sure selection appears above active line
    '.cm-selectionLayer': {
        zIndex: 100,
    },
    // Search functionality
    '.cm-searchMatch': {
        backgroundColor: '#0e639c40',
        outline: `1px solid ${base_cyan}`,
        color: base05,
        borderRadius: _utils.generalSearchField.borderRadius,
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base_cyan,
        color: background,
        padding: _utils.generalSearchField.padding,
        '& span': {
            color: background,
        },
    },
    '.cm-search.cm-panel.cm-textfield': {
        color: base05,
        borderRadius: _utils.generalSearchField.borderRadius,
        padding: _utils.generalSearchField.padding,
    },
    // Panels
    '.cm-panels': {
        backgroundColor: base06,
        color: base05,
        borderRadius: '4px',
        boxShadow: '0 1px 3px rgba(0, 0, 0, 0.1)',
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: `1px solid ${base03}30`,
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: `1px solid ${base03}30`,
    },
    '.cm-panel button': {
        backgroundColor: tooltipBackground,
        color: base05,
        border: _utils.generalPanel.border,
        borderRadius: _utils.generalPanel.borderRadius,
        padding: _utils.generalPanel.padding,
    },
    '.cm-panel button:hover': {
        backgroundColor: base07,
        border: `1px solid ${base02}`,
    },
    // Line highlighting
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
        borderRadius: _utils.generalLine.borderRadius,
        zIndex: 1,
    },
    // Gutters
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03,
        border: _utils.generalGutter.border,
        borderRight: `1px solid ${base07}`,
        paddingRight: _utils.generalGutter.paddingRight,
    },
    '.cm-activeLineGutter': {
        // backgroundColor: '#CFD0D608)',
        color: base01,
        fontWeight: _utils.generalGutter.fontWeight,
    },
    '.cm-lineNumbers': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter .cm-gutterElement': {
        color: base03,
        cursor: 'pointer',
    },
    '.cm-foldGutter .cm-gutterElement:hover': {
        color: base01,
    },
    // Tooltips and autocomplete
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
        border: `1px solid ${base04}40`,
        borderRadius: _utils.generalTooltip.borderRadius,
        padding: _utils.generalTooltip.padding,
        boxShadow: '0 2px 8px rgba(0, 0, 0, 0.15)',
    },
    '.cm-tooltip-autocomplete': {
        '& > ul': {
            backgroundColor: tooltipBackground,
            border: 'none',
            maxHeight: '300px',
        },
        '& > ul > li': {
            padding: _utils.generalTooltip.padding,
            lineHeight: _utils.generalTooltip.lineHeight,
        },
        '& > ul > li[aria-selected]': {
            backgroundColor: selection,
            color: base01,
            borderRadius: _utils.generalTooltip.borderRadiusSelected,
        },
        '& > ul > li > span.cm-completionIcon': {
            color: base03,
            paddingRight: _utils.generalTooltip.paddingRight,
        },
        '& > ul > li > span.cm-completionDetail': {
            color: base03,
            fontStyle: 'italic',
        },
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    // Diagnostics styling
    '.cm-diagnostic': {
        '&-error': {
            borderLeft: `3px solid ${invalid}`,
        },
        '&-warning': {
            borderLeft: `3px solid ${diagnosticWarning}`,
        },
        '&-info': {
            borderLeft: `3px solid ${linkColor}`,
        },
    },
    '.cm-lintPoint-error': {
        borderBottom: `2px wavy ${invalid}`,
    },
    '.cm-lintPoint-warning': {
        borderBottom: `2px wavy ${diagnosticWarning}`,
    },
    // Matching brackets
    '.cm-matchingBracket': {
        backgroundColor: activeBracketBg,
        outline: `1px solid ${activeBracketBorder}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    '.cm-nonmatchingBracket': {
        backgroundColor: `${base_red}20`,
        outline: `1px solid ${invalid}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Selection matches
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
        outline: `1px solid ${base02}50`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Fold placeholder
    '.cm-foldPlaceholder': {
        backgroundColor: tooltipBackground,
        color: base03,
        border: `1px dotted ${base03}70`,
        borderRadius: _utils.generalPlaceholder.borderRadius,
        padding: _utils.generalPlaceholder.padding,
        margin: _utils.generalPlaceholder.margin,
    },
    // Focus outline
    '&.cm-focused': {
        outline: 'none',
        boxShadow: `0 0 0 2px ${background}, 0 0 0 3px ${base_cyan}40`,
    },
    // Scrollbars
    '& .cm-scroller::-webkit-scrollbar': {
        width: _utils.generalScroller.width,
        height: _utils.generalScroller.height,
    },
    '& .cm-scroller::-webkit-scrollbar-track': {
        background: darkBackground,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb': {
        backgroundColor: base03,
        borderRadius: _utils.generalScroller.borderRadius,
        border: `3px solid ${darkBackground}`,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb:hover': {
        backgroundColor: base02,
    },
    // Ghost text
    '.cm-ghostText': {
        opacity: '0.5',
        color: base03,
    },
}, { dark: false });
/**
 * Enhanced syntax highlighting for Tokyo Night Day theme
 */
const tokyoNightDayHighlightStyle = language.HighlightStyle.define([
    // Keywords and control flow
    { tag: highlight.tags.keyword, color: base_cyan, fontWeight: 'bold' },
    { tag: highlight.tags.controlKeyword, color: base_cyan, fontWeight: 'bold' },
    { tag: highlight.tags.moduleKeyword, color: base_cyan, fontWeight: 'bold' },
    // Names and variables
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base_blue },
    { tag: [highlight.tags.variableName], color: base01 },
    { tag: [highlight.tags.propertyName], color: base_blue, fontStyle: 'normal' },
    // Classes and types
    { tag: [highlight.tags.typeName], color: base_cyan },
    { tag: [highlight.tags.className], color: base_yellow, fontStyle: 'italic' },
    { tag: [highlight.tags.namespace], color: base_purple, fontStyle: 'italic' },
    // Operators and punctuation
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base_purple },
    { tag: [highlight.tags.bracket], color: base03 },
    { tag: [highlight.tags.brace], color: base03 },
    { tag: [highlight.tags.punctuation], color: base03 },
    // Functions and parameters
    { tag: [highlight.tags.function(highlight.tags.variableName)], color: base_cyan },
    { tag: [highlight.tags.labelName], color: base_purple, fontStyle: 'italic' },
    { tag: [highlight.tags.definition(highlight.tags.function(highlight.tags.variableName))], color: base_cyan },
    { tag: [highlight.tags.definition(highlight.tags.variableName)], color: base_blue },
    // Constants and literals
    { tag: highlight.tags.number, color: base_orange },
    { tag: highlight.tags.changed, color: base_orange },
    { tag: highlight.tags.annotation, color: invalid, fontStyle: 'italic' },
    { tag: highlight.tags.modifier, color: base_orange, fontStyle: 'italic' },
    { tag: highlight.tags.self, color: base_orange },
    {
        tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)],
        color: base_orange,
    },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base_orange },
    // Strings and regex
    { tag: [highlight.tags.processingInstruction, highlight.tags.inserted], color: base_green },
    { tag: [highlight.tags.special(highlight.tags.string), highlight.tags.regexp], color: base_magenta },
    { tag: highlight.tags.string, color: base_green },
    // Punctuation and structure
    { tag: highlight.tags.definition(highlight.tags.typeName), color: base_cyan, fontWeight: 'bold' },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base_blue },
    // Comments and documentation
    { tag: highlight.tags.meta, color: base03 },
    { tag: highlight.tags.comment, fontStyle: 'italic', color: base03 },
    { tag: highlight.tags.docComment, fontStyle: 'italic', color: base03 },
    // HTML/XML elements
    { tag: [highlight.tags.tagName], color: base_purple },
    { tag: [highlight.tags.attributeName], color: base_yellow },
    // Markdown and text formatting
    { tag: [highlight.tags.heading], fontWeight: 'bold', color: base_orange },
    { tag: highlight.tags.heading1, color: base_orange, fontWeight: 'bold' },
    { tag: highlight.tags.heading2, color: base_orange },
    { tag: highlight.tags.heading3, color: base_orange },
    { tag: highlight.tags.heading4, color: base_cyan },
    { tag: highlight.tags.heading5, color: base_cyan },
    { tag: highlight.tags.heading6, color: base_cyan },
    { tag: [highlight.tags.strong], fontWeight: 'bold', color: base01 },
    { tag: [highlight.tags.emphasis], fontStyle: 'italic', color: base_green },
    // Links and URLs
    {
        tag: [highlight.tags.link],
        color: visitedLinkColor,
        fontWeight: '500',
        textDecoration: 'underline',
        textUnderlinePosition: 'under',
    },
    {
        tag: [highlight.tags.url],
        color: linkColor,
        textDecoration: 'underline',
        textUnderlineOffset: '2px',
    },
    // Special states
    {
        tag: [highlight.tags.invalid],
        color: base01,
        textDecoration: 'underline wavy',
        borderBottom: `1px wavy ${invalid}`,
    },
    { tag: [highlight.tags.strikethrough], color: invalid, textDecoration: 'line-through' },
    // Enhanced syntax highlighting
    { tag: highlight.tags.constant(highlight.tags.name), color: base_orange },
    { tag: highlight.tags.deleted, color: invalid },
    { tag: highlight.tags.squareBracket, color: base03 },
    { tag: highlight.tags.angleBracket, color: base03 },
    // Additional specific styles
    { tag: highlight.tags.monospace, color: base01 },
    { tag: [highlight.tags.contentSeparator], color: base_blue },
    { tag: highlight.tags.quote, color: base03 },
]);
/**
 * Combined Tokyo Night Day theme extension
 */
const tokyoNightDay = [
    tokyoNightDayTheme,
    language.syntaxHighlighting(tokyoNightDayHighlightStyle),
];

exports.tokyoNightDay = tokyoNightDay;
exports.tokyoNightDayHighlightStyle = tokyoNightDayHighlightStyle;
exports.tokyoNightDayTheme = tokyoNightDayTheme;
